-------------------------------------------------------------------------
--  _   _ _____ ____       ____   ___   __     __    _       _
-- | | | | ____/ ___|     / ___| / _ \  \ \   / /_ _| | __ _(_)___
-- | |_| |  _| \___ \ ____\___ \| | | |  \ \ / / _` | |/ _` | / __|
-- |  _  | |___ ___) |_____|__) | |_| |   \ V / (_| | | (_| | \__ \
-- |_| |_|_____|____/     |____/ \___/     \_/ \__,_|_|\__,_|_|___/
--
-------------------------------------------------------------------------
-- Title : Video passthrough
--
-------------------------------------------------------------------------
-- Author(s) : Marc Pignat
-------------------------------------------------------------------------
library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;

library unisim;
	use unisim.vcomponents.all;

architecture xilinx_s6 of pass_through is

	signal internal_clock						: std_logic;
	signal bufio2_sample_clock					: std_logic;
	signal bufio2_to_bufg_clock					: std_logic;

	signal data_in								: std_logic_vector(data_input'range);

	type data_internal_t is array(g_pipe_stages downto 0) of std_logic_vector(data_input'range);
	signal data_internal						: data_internal_t;

	signal data_out								: std_logic_vector(data_input'range);

	-- Force data sampling into the IO pads
	attribute iob								: string;
	attribute iob of data_in					: signal is "FORCE";
	attribute iob of data_out					: signal is "FORCE";

	-- The synthethiser should really not convert our pipeline into a shift register
	attribute shreg_extract : string;
	attribute shreg_extract of data_in			: signal is "false";
	attribute shreg_extract of data_internal	: signal is "false";
	attribute shreg_extract of data_out			: signal is "false";

begin
	----------------------------------------------------------------------------
	-- Clock mgmt for using the BUFIO2
	----------------------------------------------------------------------------
	i_bufio2 : BUFIO2
	generic map
	(
		DIVIDE			=> 1,		-- Do not divide the clock
		DIVIDE_BYPASS	=> TRUE,	-- Bypass the clock divider
		I_INVERT		=> FALSE,	-- Do not invert clock
		USE_DOUBLER		=> FALSE	-- Do not double the clock
	)
	port map
	(
		DIVCLK			=> bufio2_to_bufg_clock,-- This clock must be transmitted to a BUFG
		IOCLK			=> bufio2_sample_clock,	-- This clock must be used to sample datas on the IOBUF2
		SERDESSTROBE	=> open,				-- Unused
		I				=> clock_input			-- This is the external clock for inputs
	);

    i_bufg : BUFG
	port map
	(
		O => internal_clock,
		I => bufio2_to_bufg_clock
	);

	----------------------------------------------------------------------------
	-- Read the inputs
	-- They all are on the same IOBUF2 region
	----------------------------------------------------------------------------
	process(reset, bufio2_sample_clock)
	begin
		if reset = '1' then
			data_in <= (others => '0');
		elsif rising_edge(bufio2_sample_clock) then
			data_in	<= data_input;
		end if;
	end process;

	----------------------------------------------------------------------------
	-- Internal data (let some time for the routing delay)
	----------------------------------------------------------------------------
	process(reset, internal_clock)
	begin
		if reset = '1' then
			for i in data_internal'range loop
				data_internal(i) <= (others => '0');
			end loop;
		elsif rising_edge(internal_clock) then
			data_internal(data_internal'left) <= data_in;
			for i in data_internal'left - 1 downto data_internal'right loop
				data_internal(i) <= data_internal(i+1);
			end loop;
		end if;
	end process;

	----------------------------------------------------------------------------
	-- Spy data, from the middle of the pipe
	----------------------------------------------------------------------------
	-- spy_data <= data_internal(g_pipe_stages/2);
	-- spy_clock <= internal_clock;

	----------------------------------------------------------------------------
	-- Set the outputs
	----------------------------------------------------------------------------
	process(reset, internal_clock)
	begin
		if reset = '1' then
			data_out <= (others => '0');
		elsif rising_edge(internal_clock) then
			data_out <= data_internal(data_internal'right);
		end if;
	end process;

	data_output <= data_out;

	i_clock_out : entity work.clock_output_inverting_xilinx
	port map
	(
		clock_in		=> internal_clock,
		clock_out		=> clock_output
	);
end xilinx_s6;
